% Voir livre Perron et Munger: Analyse des systèmes et matrice d'état et exercices multi-physiques 2022 
% REMARQUE AU LECTEUR: LE MODÈLE SUGGÉRÉ EST UN EXEMPLE ACCADÉMIQUE.  
% TOUTES AUTRES APPLICATIONS EST AU RISQUE DE L'UTILISATEUR

% Problème : Régulation du niveau de liquide
close all, clear all, clc     

%% Parametres du probleme
K = 180;
rho = 1000;
A1 = 6;
A2 = 6;
q0 = 1000;

%% Question 1 : Trouver l'equation d'etat
% syms a1 a2 b1 b2 K rho A1 A2 q0
a1 = K/(rho*A1);
a2 = K/(rho*A2);
b1 = -q0/(rho*A1);
b2 = q0/(rho*A2);

matA = [-a1 a1;a2 -a2]; % Matrice d'etat
matB = [b1; b2];    % Matrice de controle

%% Question 2 : Trouver le resolvant
syms s t
Identite = eye(2);          % Matrice identité [2x2]
Phi = inv(s*Identite-matA); % Inverse la matrice 

%% Question 3 : Trouver G1(s) = h1(s)/alpha(s) et G2(s)  = h2(s)/alpha(s)
G = Phi*matB;
G1 = G(1);
G2 = G(2);

num1 = b1*[1 a2];
den1 = [1 a1+a2 0];
num2 = [a1*b2];
G1TF = tf(num1,den1)+tf(num2,den1);

num3 = [a2*b1];
num4 = b2*[1 a1];
G2TF = tf(num3,den1)+tf(num4,den1);

%% Question 4 : Trouver les pôles, ce système est-il stable?
pG1 = pole(G1TF);
zG1 = zero(G1TF);
pG2 = pole(G2TF);
zG2 = zero(G2TF);

hold on
pzmap(pG1,zG1)
grid on
hold off

hold on
pzmap(pG2,zG2)
grid on
hold off

%% Question 5 : Trouver h1(t) et h2(t)
syms s t h1_0 h2_0 alpha
h1_0 = 10;
h2_0 = 0;
alpha = 0.36;
C1 = [1 0];
C2 = [0 1];
uS = [alpha/s];
xi = [h1_0;h2_0];

h1S = C1*Phi*(xi+matB*uS);
h2S = C2*Phi*(xi+matB*uS);

h1T = ilaplace(h1S,t);
h2T = ilaplace(h2S,t);

%% Question 6 : Tracer h1(t) et h2(t)
temps = linspace(0,100,5000);
h1Taff = subs(h1T,t,temps);
h2Taff = subs(h2T,t,temps);

figure (1)
hold on
plot(temps, h1Taff)
plot(temps,h2Taff,'LineStyle','--')
xlabel('Temps (s)')
ylabel('Niveau de liquide (m)')
title('Évolution du niveau de liquide des réservoirs')
legend('Réservoir 1','Réservoir 2')
grid on
hold off

%% Partie B
%% Question 7 : Équation d'état
%% Question 1 : Trouver l'equation d'etat
% syms a1 a2 b1 b2 b3 b4 K rho A1 A2 q0
a1 = K/(rho*A1);
a2 = K/(rho*A2);
b1 = -q0/(rho*A1);
b2 = q0/(rho*A2);
b3 = 1/(rho*A1);
b4 = -1/(rho*A2);

matA_2 = [-a1 a1;a2 -a2]; % Matrice d'etat
matB_2 = [b1 b3 0;b2 0 b4];    % Matrice de controle

%% Question 8 : Trouver h1(t) et h2(t)
syms s t h1_0 h2_0 alpha qi qe
h1_0 = 10;
h2_0 = 0;
alpha = 0.36;
qi = 100;
qe = 100;
C1 = [1 0];
C2 = [0 1];

uS_2 = [alpha/s;qi/s;qe/s];
xi = [h1_0;h2_0];

h1S_2 = C1*Phi*(xi+matB_2*uS_2);
h2S_2 = C2*Phi*(xi+matB_2*uS_2);

h1T_2 = ilaplace(h1S_2,t);
h2T_2 = ilaplace(h2S_2,t);

%% Question 9 : Tracer h1(t) et h2(t)
h1Taff_2 = subs(h1T_2,t,temps);
h2Taff_2 = subs(h2T_2,t,temps);

figure (2)
hold on
plot(temps, h1Taff_2)
plot(temps,h2Taff_2,'LineStyle','--')
xlabel('Temps (s)')
ylabel('Niveau de liquide (m)')
title('Évolution du niveau de liquide des réservoirs')
legend('Réservoir 1','Réservoir 2')
grid on
hold off

%% Question 10 : Trouver h1(inf) et h2(inf)
h1Taff_2_inf = subs(h1T_2,t,inf);
h2Taff_2_inf = subs(h2T_2,t,inf);

%% Question 11 : Trouver H(s)
syms P% rho A2
P = 500;
Gc = P;
Gp = 1/(s*rho*A2);
Hs = Gc*Gp/(1+(Gc*Gp));

%% Question 13 : Trouver h1(t), h2(t) et alpha(t) en boucle fermée
syms h2_ref
qe = 0;
qi = 0;
h2_ref = 6;
h2S_3 = Hs*h2_ref/s+h2_0/s;
h2T_3 = ilaplace(h2S_3,t);

h1S_3 = (qi/s-(s*rho*A2*h2S_3)+qe/s)/(s*rho*A1)+h1_0/s;
h1T_3 = ilaplace(h1S_3,t);

alphaS_3 = (s*rho*A2*h2S_3 + qe/s + K*(h2S_3-h1S_3))/(q0);
alphaT_3 = ilaplace(alphaS_3,t);

temps = linspace(0,150,1000);

h1Taff_3 = subs(h1T_3,t,temps);
h2Taff_3 = subs(h2T_3,t,temps);
alphaTaff_3_temp = subs(alphaT_3,t,temps);
alphaTaff_3 = max(min(alphaTaff_3_temp,1),0);

figure (3)
hold on
plot(temps, h1Taff_3)
plot(temps, h2Taff_3,'LineStyle','--')
xlabel('Temps (s)')
ylabel('Niveau de liquide (m)')
title('Évolution du niveau de liquide des réservoirs')
legend('Réservoir 1','Réservoir 2')
grid on
hold off

figure (4)
hold on
plot(temps, alphaTaff_3)
xlabel('Temps (s)')
ylabel('\alpha (-)')
title('Évolution de \alpha de la pompe')
grid on
hold off

%% Question 14 : Tracer h1(t) et h2(t) en boucle fermée
temps2 = linspace(0,50,1000);
qe = 200;
qi = 0;
h2_ref = 6;
% 150 - 200 (s)
h2S_4 = h2_ref/s;
h2T_4 = ilaplace(h2S_4,t);

h1S_4 = (qi/s-(s*rho*A2*h2S_4)-qe/s)/(s*rho*A1)+h1_0/s;
h1T_4 = ilaplace(h1S_4,t);

alphaS_4 = (s*rho*A2*h2S_4 - qe/s + K*(h2S_4-h1S_4))/(q0);
alphaT_4 = ilaplace(alphaS_4,t);

h1Taff_4 = subs(h1T_4,t,temps2);
h2Taff_4 = subs(h2T_4,t,temps2);
alphaTaff_4_temp = subs(alphaT_4,t,temps2);
alphaTaff_4 =max(min(alphaTaff_4_temp,1),0);

% 200 - 250 (S)
qe = 0;
qi = 0;
h2_ref = 6;

h2S_5 = h2_ref/s;
h2T_5 = ilaplace(h2S_5,t);

h1S_5 = h1Taff_4(end)/s;
h1T_5 = ilaplace(h1S_5,t);

alphaS_5 = (s*rho*A2*h2S_5 - qe/s + K*(h2S_5-h1S_5))/(q0);
alphaT_5 = ilaplace(alphaS_5,t);

h1Taff_5 = subs(h1T_5,t,temps2);
h2Taff_5 = subs(h2T_5,t,temps2);
alphaTaff_5_temp = subs(alphaT_5,t,temps2);
alphaTaff_5 = max(min(alphaTaff_5_temp,1),0);

temps_1 = temps;
temps_2 = linspace(150,200,1000);
temps_3 = linspace(200,250,1000);

figure (5)
hold on
plot([temps_1 temps_2 temps_3], [h1Taff_3 h1Taff_4 h1Taff_5])
plot([temps_1 temps_2 temps_3], [h2Taff_3 h2Taff_4 h2Taff_5],'LineStyle','--')
xlabel('Temps (s)')
ylabel('Niveau de liquide (m)')
title('Évolution du niveau de liquide des réservoirs')
legend('Réservoir 1','Réservoir 2')
grid on
hold off

%% Question 15 : Trouver h2(inf) maximal
alpha = 1;
uS = [alpha/s];
xi = [h1_0;h2_0];

h1S = C1*Phi*(xi+matB*uS);
h2S = C2*Phi*(xi+matB*uS);

h1T = ilaplace(h1S,t);
h2T = ilaplace(h2S,t);

temps = linspace(0,100,5000);
h1Taff = subs(h1T,t,temps);
h2Taff = subs(h2T,t,inf);

figure (6)
hold on
plot(temps, h1Taff)
plot(temps,h2Taff,'LineStyle','--')
xlabel('Temps (s)')
ylabel('Niveau de liquide (m)')
title('Évolution du niveau de liquide des réservoirs')
legend('Réservoir 1','Réservoir 2')
grid on
hold off

%% Question 16 : Selon la fonction de transfert G1(s). Avons-nous un filtre passe-bas ou passe-haut?
omega = linspace(0.01,1000,1000);
G1omega = subs(G1,s,j*omega);
G1aff = abs(G1omega);

semilogx(G1aff, omega);
xlabel('Pulsation (rad/s)')
ylabel('G_1(\omega)')
title('Réponse fréquentielle de la fonction de transfert G_1(s)')
grid on

%% Question 17 : Si on désire obtenir un niveau h2 = 8.5 m trouver q0.
syms q0
b1 = -q0/(rho*A1);
b2 = q0/(rho*A2);
matA = [-a1 a1;a2 -a2]; % Matrice d'etat
matB = [b1; b2];    % Matrice de controle
Phi = inv(s*Identite-matA); % Inverse la matrice 

alpha = 1;
uS = [alpha/s];
xi = [h1_0;h2_0];
h2S = C2*Phi*(xi+matB*uS);
h2T = ilaplace(h2S,t);

h2Tinf = subs(h2T,t,inf);
s = solve(h2Tinf==8.5,q0);